@@ -1,7 +1,19 @@
-import types
+import math, types
 import numpy as N
 import matplotlib.pyplot as P
 
+def plotres(psr):
+    """Plot residuals, compute unweighted rms residual."""
+    
+    res = math.sqrt(N.mean(psr.residuals()**2)) / 1e-6
+
+    t = psr.toas(); i = N.argsort(t)
+    
+    P.errorbar(t[i],psr.residuals()[i]/1e-6,yerr=psr.toaerrs[i],fmt='x')
+    
+    P.xlabel('MJD'); P.ylabel('res [us]')
+    P.title("{0} - rms res = {1:.2f} us".format(psr.name,res))
+
 # select parameters by name or number, omit non-existing
 def _select(p,pars,select):
     sel = []
@@ -12,7 +24,7 @@ def _select(p,pars,select):
             sel.append(s)
     return len(sel), sel
 
-def plothist(data,pars=[],offsets=[],norms=[],select=[],weights={},ranges={},append=False,
+def plothist(data,pars=[],offsets=[],norms=[],select=[],weights={},ranges={},labels={},skip=[],append=False,
              bins=50,color='k',linestyle=None,title=None):
     if hasattr(data,'data') and not isinstance(data,N.ndarray):
         # parse a multinestdata structure
@@ -26,13 +38,21 @@ def plothist(data,pars=[],offsets=[],norms=[],select=[],weights={},ranges={},app
         pars = map('p{0}'.format,range(p))
 
     if offsets:
-        if len(offsets) < p:
-            offsets = offsets + [0.0] * (p - len(offsets))
-        data = data - N.array(offsets)
+        data = data.copy()
+
+        if isinstance(offsets,dict):
+            for i,par in enumerate(pars):
+                if par in offsets:
+                    data[:,i] = data[:,i] - offsets[par]
+        else:
+            if len(offsets) < p:
+                offsets = offsets + [0.0] * (p - len(offsets))
+            data = data - N.array(offsets)
 
     if norms:
         if len(norms) < p:
             norms = norms + [1.0] * (p - len(norms))
+
         data = data / norms
 
     if select:
@@ -59,7 +79,13 @@ def plothist(data,pars=[],offsets=[],norms=[],select=[],weights={},ranges={},app
     for i in range(p):
         # need to do this since isinstance(False,int) == True
         q = append if type(append) == int else p
-        P.subplot(int((q-1)/4)+1,min(q,4),i+1)
+        
+        sp = i + 1
+        for s in skip:
+            if i >= s:
+                sp = sp + 1
+
+        P.subplot(int((q-1)/4)+1,min(q,4),sp)
 
         if append:
             P.hold(True)
@@ -67,14 +93,27 @@ def plothist(data,pars=[],offsets=[],norms=[],select=[],weights={},ranges={},app
         if pars[i] in ranges:
             dx = ranges[pars[i]]
             P.hist(data[:,i],bins=int(bins * (N.max(data[:,i]) - N.min(data[:,i])) / (dx[1] - dx[0])),
-                   weights=weight,normed=True,histtype='step',color=color,linestyle=linestyle)
+                   weights=weight,normed=True,histtype='step',color=color,linestyle=linestyle,linewidth=1)
             P.xlim(dx)
         else:
             P.hist(data[:,i],bins=bins,
-                   weights=weight,normed=True,histtype='step',color=color,linestyle=linestyle)
+                   weights=weight,normed=True,histtype='step',color=color,linestyle=linestyle,linewidth=1)
+
+        P.xlabel(labels[pars[i]] if pars[i] in labels else pars[i],fontsize=13)
+
+        # P.ticklabel_format(style='sci',axis='both',scilimits=(-3,4),useoffset='True')
+        P.locator_params(axis='both',nbins=6)
+        P.minorticks_on()
+
+        fx = P.ScalarFormatter(useOffset=True,useMathText=True)
+        fx.set_powerlimits((-3,4)); fx.set_scientific(True)
+
+        fy = P.ScalarFormatter(useOffset=True,useMathText=True)
+        fy.set_powerlimits((-3,4)); fy.set_scientific(True)
+
+        P.gca().xaxis.set_major_formatter(fx)
+        P.gca().yaxis.set_major_formatter(fy)
 
-        P.xlabel(pars[i])
-        P.ticklabel_format(style='sci',axis='x',scilimits=(-2,2),useoffset='True')
         P.hold(False)
 
     if title and not append:
@@ -84,12 +123,13 @@ def plothist(data,pars=[],offsets=[],norms=[],select=[],weights={},ranges={},app
 
 # to do: should fix this histogram so that the contours are correct
 #        even for restricted ranges...
-def _plotonehist2(x,y,parx,pary,smooth,ranges={},bins=50,weights=None):
+def _plotonehist2(x,y,parx,pary,smooth,colormap=True,ranges={},labels={},bins=50,weights=None):
     hrange = [ranges[parx] if parx in ranges else [N.min(x),N.max(x)],
               ranges[pary] if pary in ranges else [N.min(y),N.max(y)]]
 
     [h,xs,ys] = N.histogram2d(x,y,bins=bins,normed=True,range=hrange,weights=weights)
-    P.contourf(0.5*(xs[1:]+xs[:-1]),0.5*(ys[1:]+ys[:-1]),h.T,cmap=P.get_cmap('YlOrBr')); P.hold(True)
+    if colormap:
+        P.contourf(0.5*(xs[1:]+xs[:-1]),0.5*(ys[1:]+ys[:-1]),h.T,cmap=P.get_cmap('YlOrBr')); P.hold(True)
 
     H,tmp1,tmp2 = N.histogram2d(x,y,bins=bins,range=hrange,weights=weights)
 
@@ -110,17 +150,18 @@ def _plotonehist2(x,y,parx,pary,smooth,ranges={},bins=50,weights=None):
     xs = N.linspace(hrange[0][0],hrange[0][1],bins)
     ys = N.linspace(hrange[1][0],hrange[1][1],bins)
 
-    P.contour(xs,ys,H.T,levels,colors='k',linestyles=('-','--','-.'),linewidths=2); P.hold(False)
+    P.contour(xs,ys,H.T,levels,colors='k',linestyles=('-','--','-.'),linewidths=1); P.hold(False)
 
     if parx in ranges:
         P.xlim(ranges[parx])
     if pary in ranges:
         P.ylim(ranges[pary])
 
-    P.xlabel(parx); P.ylabel(pary)
+    P.xlabel(labels[parx] if parx in labels else parx,fontsize=13)
+    P.ylabel(labels[pary] if pary in labels else pary,fontsize=13)
     P.ticklabel_format(style='sci',axis='both',scilimits=(-2,2),useoffset='True')
 
-def plothist2(data,pars=[],offsets=[],smooth=False,select=[],ranges={},bins=50,diagonal=True,title=None,append=False):
+def plothist2(data,pars=[],offsets=[],smooth=False,colormap=True,select=[],ranges={},labels={},bins=50,weights=None,diagonal=True,title=None,append=False):
     if hasattr(data,'data') and not isinstance(data,N.ndarray):
         # parse a multinestdata structure
         if not pars    and hasattr(data,'parnames'):
@@ -141,6 +182,17 @@ def plothist2(data,pars=[],offsets=[],smooth=False,select=[],ranges={},bins=50,d
         m, sel = _select(m,pars,select)
         data, pars = data[:,sel], [pars[s] for s in sel]
 
+    if weights:
+        weight = 1
+        for i,par in enumerate(pars):
+            if par in weights:
+                if isinstance(weights[par],types.FunctionType):
+                    weight = weight * N.vectorize(weights[par])(data[:,i])
+                else:
+                    weight = weight * weights[par]
+    else:
+        weight = None
+
     if not append:
         fs = min((m if diagonal else m-1)*4,16)
         P.figure(figsize=(fs,fs))
@@ -155,26 +207,27 @@ def plothist2(data,pars=[],offsets=[],smooth=False,select=[],ranges={},bins=50,d
             if pars[i] in ranges:
                 dx = ranges[pars[i]]
                 P.hist(data[i],bins=int(50 * (N.max(data[i]) - N.min(data[i])) / (dx[1] - dx[0])),
-                       normed=True,histtype='step',color='k')
+                       weights=weight,normed=True,histtype='step',color='k')
                 P.xlim(dx)
             else:
-                P.hist(data[i],bins=50,normed=True,histtype='step',color='k')
+                P.hist(data[i],bins=50,weights=weight,normed=True,histtype='step',color='k')
 
-            P.xlabel(pars[i])
+            P.xlabel(labels[pars[i]] if pars[i] in labels else pars[i],fontsize=13)
             P.ticklabel_format(style='sci',axis='both',scilimits=(-2,2),useoffset='True')
+            P.tick_params(labelsize=12)
 
             for j in range(0,i):
                 if not append:
                     P.subplot(m,m,i*m+j+1)
 
-                _plotonehist2(data[j],data[i],pars[j],pars[i],smooth,ranges,bins)
+                _plotonehist2(data[j],data[i],pars[j],pars[i],smooth,colormap,ranges,labels,bins,weights=weight)
     else:
         for i in range(m-1):
             for j in range(i+1,m):
                 if not append:
                     P.subplot(m-1,m-1,(m-1)*i+j)
 
-                _plotonehist2(data[j],data[i],pars[j],pars[i],smooth,ranges,bins)
+                _plotonehist2(data[j],data[i],pars[j],pars[i],smooth,colormap,ranges,labels,bins,weights=weight)
 
     P.tight_layout()
 
